@@ -1,7 +1,7 @@
 from typing import Callable, Type, Any, Union, Tuple
 import functools
 from .Functions import areoneof, isoneof, isoneof_strict
-from .Exceptions import OverloadDuplication, OverloadNotFound, ValidationTypeError, ValidationValueError
+# from .Exceptions import OverloadDuplication, OverloadNotFound, ValidationTypeError, ValidationValueError
 
 
 def NotImplemented(func: Callable) -> Callable:
@@ -47,6 +47,9 @@ def memo(func: Callable) -> Callable:
     return wrapper
 
 
+__validation_set = set()
+
+
 def validate(*args) -> Callable:
     """validate decorator
 
@@ -60,10 +63,19 @@ def validate(*args) -> Callable:
         4. Sequence that contains three arguments:\n
             4.1 a Type or Sequence[Type]\n
             4.2 a function to call on argument\n
-            4.3 a str to display in a Value error iff the condition from 4.2 fails\n
+            4.3 a str to display in a ValueError iff the condition from 4.2 fails\n
     """
+    from .Exceptions import ValidationDuplicationError, ValidationTypeError, ValidationValueError
 
     def wrapper(func: Callable) -> Callable:
+        global __validation_set
+        func_id = f"{func.__module__}.{func.__name__}"
+        if func_id not in __validation_set:
+            __validation_set.add(func_id)
+        else:
+            raise ValidationDuplicationError(
+                "validate decorator is being used on two functions in the same module with the same name\nmaybe use @overload instead")
+
         def validate_type(v: Any, T: Type, validation_func: Callable[[Any], bool] = isinstance) -> None:
             if not validation_func(v, T):
                 raise ValidationTypeError(
@@ -107,6 +119,155 @@ def validate(*args) -> Callable:
     return wrapper
 
 
+__overload_dict: dict[str, dict[Tuple, Callable]] = dict()
+
+
+def overload(*types) -> Callable:
+    """decorator for overloading functions\n
+    Usage\n-------\n
+    @overload(str,str)\n
+    def print_name_and_color(name,color):
+        ...\n\n
+    @overload(str,[int,float]))\n
+    def print_name_and_age(name,age):
+        ...\n\n
+
+    also, use 'None' to skip\n\n\n
+    \nRaises:
+        OverloadDuplication: if a functions is overloaded twice (or more) with same argument types
+        OverloadNotFound: if an overloaded function is called with types that has no variant of the function
+
+    \nNotice:
+        The function's __doc__ will hold the value of the last variant only
+    """
+    from .Exceptions import OverloadDuplication, OverloadNotFound
+    # make sure to use uniqe global dictonary
+    global __overload_dict
+
+    # allow input of both tuples and lists for flexabily
+    types = list(types)
+    for i, maybe_list_of_types in enumerate(types):
+        if isoneof(maybe_list_of_types, [list, Tuple]):
+            types[i] = tuple(sorted(list(maybe_list_of_types),
+                             key=lambda sub_type: sub_type.__name__))
+    types = tuple(types)
+
+    def wrapper(func: Callable) -> Callable:
+
+        # assing current overload to overload dictionary
+        name = f"{func.__module__}.{func.__name__}"
+
+        if name not in __overload_dict:
+            __overload_dict[name] = dict()
+
+        if types in __overload_dict[name]:
+            # raise if current overload already exists for current function
+            raise OverloadDuplication(
+                f"{name} has duplicate overloading for type(s): {types}")
+
+        __overload_dict[name][types] = func
+
+        @functools.wraps(func)
+        def inner(*args, **kwargs) -> Any:
+
+            # select corret overload
+            for variable_types, curr_func in __overload_dict[f"{func.__module__}.{func.__name__}"].items():
+                if len(variable_types) != len(args):
+                    continue
+
+                for i, variable_type in enumerate(variable_types):
+                    if variable_type is not None:
+                        if isoneof(variable_type, [list, Tuple]):
+                            if not isoneof_strict(args[i], variable_type):
+                                break
+                        else:
+                            if not isinstance(args[i], variable_type):
+                                break
+                else:
+                    return curr_func(*args, **kwargs)
+
+            # or raise exception if no overload exists for current arguments
+            raise OverloadNotFound(
+                f"function {func.__module__}.{func.__name__} is not overloaded with {[type(v) for v in args]}")
+
+        return inner
+    return wrapper
+
+
+def abstractmethod(func: Callable) -> Callable:
+    """A decorator to mark a function to be 'pure vitual' / 'abstract'
+
+    Args:
+        func (Callable): the function to mark
+
+    Raises:
+        NotImplementedError: the error that will rise when the marked function will be called if not overriden in a derived class
+    """
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        raise NotImplementedError(
+            f"{func.__module__}.{func.__name__} MUST be overrided in a child class")
+    return wrapper
+
+
+purevirtual = abstractmethod
+
+__virtualization_tables = dict()
+
+
+@NotImplemented
+def virtual(func: Callable) -> Callable:
+    def wrapper(*args, **kwargs):
+        return func(*args, **kwargs)
+    return wrapper
+
+
+@NotImplemented
+def override(func: Callable) -> Callable:
+    def wrapper(*args, **kwargs):
+        return func(*args, **kwargs)
+    return wrapper
+
+
+@validate([[str, Callable], None, None])
+def deprecate(obj: Union[str, Callable] = None) -> Callable:
+    """decorator to mark function as depracated
+
+    Args:
+        obj (Union[str, None, Callable], optional): Defaults to None.
+
+        Can operate in two configurations:\n
+        1. obj is the function that you want to depracate\n
+        \t@deprecate\n
+        \tdef foo(...):\n
+        \t\t...\n\n
+        2. obj is an advise message\n
+        \t@deprecate("instead use ...")\n
+        \tdef foo(...):
+        \t\t...
+    """
+    # if callable(obj):
+    if isinstance(obj, Callable):
+        @functools.wraps(obj)
+        def inner(*args, **kwargs) -> Any:
+            print(
+                f"As marked by the developer, {obj.__module__}.{obj.__name__} is deprecated")
+            return obj(*args, **kwargs)
+        return inner
+
+    @validate(Callable)
+    def wrapper(func: Callable) -> Callable:
+        @functools.wraps(func)
+        def inner(*args, **kwargs) -> Any:
+            print(
+                f"As marked by the developer, {func.__module__}.{func.__name__} is deprecated")
+            if obj:
+                print(obj)
+            return func(*args, **kwargs)
+        return inner
+    return wrapper
+
+
 # @PartallyImplemented
 # @validate(str, Type, bool, Callable, str)
 # def opt(opt_name: str, opt_type: Type, is_required: bool = True, constraints: Callable[[Any], bool] = None, constraints_description: str = None) -> Callable:
@@ -134,124 +295,3 @@ def validate(*args) -> Callable:
 #             return func(*args, **kwargs)
 #         return inner
 #     return wrapper
-
-
-__overload_func_dict: dict[str, dict[Tuple, Callable]] = dict()
-
-
-@PartallyImplemented
-def overload(*types) -> Callable:
-    """decorator for overloading functions
-
-    Raises:
-        OverloadDuplication: if a functions is overloaded twice the same
-        OverloadNotFound: if an overloaded function is called with types that has no variant of the function
-    """
-    # make sure to use uniqe global dictonary
-    global __overload_func_dict
-
-    # allow input of both tuples and lists for flexabily
-    types = list(types)
-    for i, maybe_list_of_types in enumerate(types):
-        if isoneof(maybe_list_of_types, [list, Tuple]):
-            types[i] = tuple(sorted(list(maybe_list_of_types),
-                             key=lambda sub_type: sub_type.__name__))
-    types = tuple(types)
-
-    def wrapper(func: Callable) -> Callable:
-        name = f"{func.__module__}.{func.__name__}"
-
-        if name not in __overload_func_dict:
-            __overload_func_dict[name] = dict()
-
-        if types in __overload_func_dict[name]:
-            raise OverloadDuplication(
-                f"{name} already has an overload with {types}")
-
-        __overload_func_dict[name][types] = func
-
-        @functools.wraps(func)
-        def inner(*args, **kwargs) -> Any:
-
-            for variable_types, curr_func in __overload_func_dict[f"{func.__module__}.{func.__name__}"].items():
-                if len(variable_types) != len(args):
-                    continue
-
-                for i, variable_type in enumerate(variable_types):
-                    if isoneof(variable_type, [list, Tuple]):
-                        if not isoneof_strict(args[i], variable_type):
-                            break
-                    else:
-                        if not isinstance(args[i], variable_type):
-                            break
-                else:
-                    return curr_func(*args, **kwargs)
-
-            raise OverloadNotFound(
-                f"function {func.__module__}.{func.__name__} is not overloaded with {types}")
-
-        return inner
-    return wrapper
-
-
-def abstractmethod(func: Callable) -> Callable:
-    """A decorator to mark a function to be 'pure vitual' / 'abstract'
-
-    Args:
-        func (Callable): the function to mark
-
-    Raises:
-        NotImplementedError: the error that will rise when the marked function will be called if not overriden in a derived class
-    """
-    @functools.wraps(func)
-    def wrapper(*args, **kwargs):
-        raise NotImplementedError(
-            f"{func.__module__}.{func.__name__} MUST be overrided in a child class")
-    return wrapper
-
-
-@NotImplemented
-def override(func: Callable) -> Callable:
-    pass
-
-    def wrapper(*args, **kwargs):
-        return func(*args, **kwargs)
-    return wrapper
-
-
-@validate([[str, Callable], None, None])
-def deprecate(obj: Union[str, Callable] = None) -> Callable:
-    """decorator to mark function as depracated
-
-    Args:
-        obj (Union[str, None, Callable], optional): Defaults to None.
-
-        Can operate in two configurations:\n
-        1. obj is the function that you want to depracate\n
-        \t@deprecate\n
-        \tdef foo(...):\n
-        \t\t...\n\n
-        2. obj is an advise message\n
-        \t@deprecate("instead use ...")\n
-        \tdef foo(...):
-        \t\t...
-    """
-    if callable(obj):
-        @functools.wraps(obj)
-        def inner(*args, **kwargs) -> Any:
-            print(
-                f"As marked by the developer, {obj.__module__}.{obj.__name__} is deprecated")
-            return obj(*args, **kwargs)
-        return inner
-
-    @validate(Callable)
-    def wrapper(func: Callable) -> Callable:
-        @functools.wraps(func)
-        def inner(*args, **kwargs) -> Any:
-            print(
-                f"As marked by the developer, {func.__module__}.{func.__name__} is deprecated")
-            if obj:
-                print(obj)
-            return func(*args, **kwargs)
-        return inner
-    return wrapper
